package BotCore;
# $Header: /cvsroot/tuxaator/tuxaator/BotCore.pm,v 1.12 2005/02/11 20:27:36 digg Exp $
# this is the core of the bot, it loads the config file and connects
# to the IRC server, as well to the database and provides all it's sub
# classes methods to interact with those.
use strict;
use Net::IRC;
use DBI;
use Data::Dumper;
use Text::Wrapper;
use vars qw(@ISA);
@ISA = qw(Net::IRC);

sub new {
	my $proto = shift;
	my $basedir = shift;
	my $class = ref($proto) || $proto;
	my %config;
	load_config($basedir,\%config);

	my %messages = &load_messages($basedir,$config{'locale'});

	$config{'basedir'} = $basedir;
	my %self;

	$self{"basedir"} = $basedir;
	$self{"config"} = \%config;
	$self{"cfg"} = \%config;
	$self{"msgs"} = \%messages;

	$self{"plugins"} = {};
	$self{"msg_handlers"} = {};
	printf "Connecting to database server on %s...\n",$config{'db_host'};
	my $dbh = DBI->connect(sprintf("DBI:%s:database=%s;host=%s","mysql",$config{'db_base'},$config{'db_host'}),$config{'db_user'},$config{'db_pass'},{PrintError => 1});
	$dbh->{mysql_auto_reconnect} = 1;
	print "DB done\n";
	$self{"dbh"} = $dbh;
	$self{"queue"} = ();
	$self{"queued"} = 0;
	bless \%self, $class;
}

# given a hostmask and level checks whether there is an user matching
# those and if that users level is greater than the one specified as an
# argument
sub can_execute {
	my $class = shift;
	my $hostmask = shift;
	my $level = shift;
	my $allowed = 0;
	my ($key,$value);
	# since regexps are allowed, there is no other way to 
	# do this check

	# and just perhaps I should figure out a new way to 
	# do user acl checks
	while (($key, $value) = each %{$class->{"users"}})
	{
		$allowed = 1 if ( ($hostmask =~ /^\~?$key$/i) && ($value >= $level) );
	};
	if ($allowed)
	{
		print "ACCESS GRANTED\n";
	}
	else
	{
		print "ACCESS DENIED\n";
	};
	return $allowed;
}

# loads the configuration file for the bot
sub load_config {
	my ($basedir, $config) = @_;
	print "starting config load\n";
	my $cfg = sprintf("%s/config/config.tux",$basedir);
	my ($key,$value);
	open CFG,"<" . $cfg or die "cannot open configuration file\n"; 
	while(<CFG>)
	{
		unless (/^#/)
		{
			($key,$value) = $_ =~ /^\s*(.+?)\s*=\s*(.+?)$/;
			if ($key && $value)
			{
				$$config{$key} = $value;
			};
		};
	};
	close CFG;
	#return \%config;
}

sub load_plugin_config {
	print "loading plugin config!\n";
}

sub load_messages {
	my $basedir = shift;
	my $lang = shift;
	my %messages;
	print "basedir = $basedir, lang = $lang\n";
	my $fname = sprintf("%s/config/messages.tux.en",$basedir);
	return unless (-e $fname && -r $fname);
	open CFG,"<$fname" or die "$fname: $!\n";
	while (<CFG>)
	{
		unless (/^#/)
		{
			my ($key,$value) = $_ =~ /^\s*(.+?)\s*=\s*(.+?)$/;
			if ($key && $value)
			{
				#print "loaded: $key = $value\n";
				$messages{$key} = $value;
			};
		};
	};
	close CFG;
	$fname = sprintf("%s/config/messages.tux.%s",$basedir,$lang);
	open CFG,"<$fname" or die "$!\n";
	while (<CFG>)
	{
		unless (/^#/)
		{
			my ($key,$value) = $_ =~ /^\s*(.+?)\s*=\s*(.+?)$/;
			if ($key && $value)
			{
				#print "overloaded: $key = $value\n";
				$messages{$key} = $value;
			};
		};
	};
	close CFG;
	return %messages;
}

# this one can be called from inside a plugin
sub load_messages2 {
	my $class = shift;
	my $lang = shift;
	my %messages = &load_messages($class->{'config'}->{'basedir'},$lang);
	$class->{'messages'} = \%messages;
}
	

sub queue {
	my $class = shift;
	my $event = shift;	# copy of the event object for further processing
	my $msg = shift;	# what to send
	my $type = shift;	# what method to use to send data (right now "emote" triggers "me"
						#		instead of privmsg
        # Wrap the text into lines of no more than 400 characters each
		my $wrapper = new Text::Wrapper(columns => 400);
		for my $line (split /\n/, $wrapper->wrap($msg)) {
			my $rec = {};
			my $to = @{$event->to}[0];
			#my $target = (substr($to,0,1) eq "#") ? $class->{'config'}->{'channel'} : $event->{'nick'};
			my $target = (substr($to,0,1) eq "#") ? $to : $event->{'nick'};
			$rec->{'target'} = $target;
			$rec->{'msg'} = $line;
			push @{$class->{'queue'}},$rec;
			$class->{'queued'}++;
		}
};

sub get_queue_element {
	my $class = shift;
	if ($class->{'queued'} > 0)
	{
		return shift @{$class->{'queue'}};
	}
	else
	{
		#print "nothing in the queue, skipping\n";
		return {};
	}
};

# registers a plugin with the core object
sub register_plugin {
	my $self = shift;
	my ($id,$pluginref) = @_;
	if ($pluginref->can("get_msg_handlers"))
	{
		# fetch references to all msg handlers
		my %handlers = $pluginref->get_msg_handlers();
		# register individual handlers
		foreach my $handler (keys %handlers)
		{
			print "$handler ";
			# add a pointer to the module
			$handlers{$handler}->{'id'} = $id;
			$self->register_msg_handler($handler,%handlers->{$handler});
		};
	};
	# we need to store the command names somewhere
	$self->{'plugins'}->{$id} = $pluginref;
};

# registers a message handler
sub register_msg_handler {
	my $self = shift;
	my ($action,$coderef) = @_;
	$self->{'msg_handlers'}->{$action} = $coderef;
};

# tries to execute a command from the Core object or another plugin
sub execute_action {
	my ($self,$command,$event,@params) = @_;
	my $id = $self->{'msg_handlers'}->{$command}->{'id'};
	my $thiscommand = $self->{'msg_handlers'}->{$command};
	unless (eval "\$self->{'plugins'}->{\$id}->$thiscommand->{'action'}(\@params,\$event)")
	{
		if (length($@) > 0)
		{
			print "Core eval returned an error: $@\n";
		};
	};	
};


1;
