package Core;
# Core plugin
# $Header: /cvsroot/tuxaator/tuxaator/Plugins/Core/Plugin.pm,v 1.7 2005/04/15 08:46:51 digg Exp $
sub new
{
	my $class = shift;
	my $core = shift;
	my $self =
	{
		core => $core,
	};
	bless $self, $class;
}

sub get_id
{
	my $class = shift;
	return "Core";
}

sub get_msg_handlers
{
	my $class = shift;
	return ( 
		"!rehash" =>
			{
				"action"	=> "rehash_hosts",
				"restricted"	=> 1,
				"level" 	=> 9,
			},

		"!quit"	=>
			{
				"action"	=> "do_quit",
				"restricted"	=> 1,
				"level"		=> 9,
			},

		"!help"	=> 
			{
				"action"	=> "do_help",
				"restricted"	=> 0,
			},

		"!rename" => 
			{
				"action"	=> "do_rename",
				"restricted"	=> 1,
				"level"		=> 9,
			},
		
		"!join"	=> 
			{
				"action"	=> "do_join",
				"restricted"	=> 1,
				"level" 	=> 9,
			},
		"!language" => 
			{
				"action"	=> "do_language",
				"restricted"	=> 1,
				"level" 	=> 9,
			},

	);

};

sub get_hooks
{
	return
	{
		"376" => \&on_connect, # end of motd
		"422" => \&on_connect, # no motd
		"433" => \&on_nick_taken,
		"cping" => \&on_ping,
		"cversion" => \&on_cversion,
	};
}

sub on_connect
{
	my($self,$event) = @_;
	my $basedir = $self->{'Core'}->{'basedir'};
	my %users = &load_hosts($basedir, $self->{'Core'}->{'config'});
	$self->{'Core'}->{'users'} = \%users;
}

# Change our nick if someone stole it.
sub on_nick_taken
{
	my ($self) = shift;
	my @nicks = qw(Tuxaator Przwinkis Pingviin Tux Tux1 Tux2 Tux3 Tux4 Tux5 Lammas Aff);
	my $nick = $nicks[rand scalar @nicks];
	$self->{'Core'}->{'connection'}->nick($nick);
	$self->{'Core'}->{'config'}->{'nick'} = $nick;
}

sub do_help
{
	my $self = shift;
	my $event = pop @_;
	my ($key,$val);
	my @commands;
	while(($key,$val) = each %{$self->{'core'}->{'msg_handlers'}})
	{
		push @commands,$key;
	}
	$self->{'core'}->queue($event,"Ksud: " . join(", ",sort @commands));
};

sub do_rename
{
	my $self = shift;
	my $event = pop @_;
	my $newnick = shift;
	return unless($newnick);
	my $nick = $self->{'core'}->{'config'}->{'nick'};
	# it would be nice if we could check whether the nick change succeeded or not
	# but sadly we can't.
	$self->{'core'}->{'connection'}->nick($newnick);
	$self->{'core'}->{'config'}->{'nick'} = $newnick;
};

sub do_join
{
	my $self = shift;
	my $event = pop @_;
	my $newchannel = shift;
	my $channelkey = shift; # optional channel key
	return unless($newchannel);
	my $channel = $self->{'core'}->{'config'}->{'channel'};
	# doesn't work on multiple channels at once yet, part the current one
	$self->{'core'}->{'connection'}->part($channel);
	# can't check whether the join succeeded or not
	$self->{'core'}->{'connection'}->join($newchannel,$channelkey);
	$self->{'core'}->{'config'}->{'channel'} = $newchannel;
};

sub do_language
{
	my $self = shift;
	my $lang = shift;
	my $event = pop @_;
	unless ($lang =~ /\w{2}/)
	{
		$self->{'core'}->queue($event,$self->{'core'}->{'messages'}->{'language_error'});
	}
	else
	{
		$self->{'core'}->load_messages2($lang);
		$self->{'core'}->queue($event,$self->{'core'}->{'messages'}->{'language_success'});
	};
};

sub rehash_hosts
{
	my $self = shift;
	my $event = pop @_;
	my $basedir = $self->{'core'}->{'basedir'};
	my %users = &load_hosts($basedir, $self->{'core'}->{'config'});
	print "rehashing hosts\n";
	$self->{'core'}->{'users'} = \%users;
	if ($event)
	{
		$self->{'core'}->queue($event,"Done");
	};
}

# loads access control list from a file
sub load_hosts
{
	my ($basedir, $config) = @_;
	my $extra_hosts_files = '';
	$extra_hosts_files = $config->{'extra_hosts_files'} if $config;
	my %users;
	for my $hostsfile ("$basedir/config/hosts",
                           split /\s*,\s*/, $extra_hosts_files)
	{
		print STDERR "DEBUG: loading $hostsfile\n";
		unless (open HOSTS, $hostsfile) {
			warn "$hostsfile: cannot open: $!\n";
			next;
		}
		while (<HOSTS>)
		{
			chomp;
			next if m/^#/ or m/^\s*$/;
			if (m/^\s*(.+?)\s*([|=])\s*(\d+?)\s*$/)
			{
				my ($hostmask, $delim, $level) = ($1, $2, $3);
				if ($delim eq '=') # glob ?
				{
					$hostmask =~ s/([^\w*])/\\$1/g;
					$hostmask =~ s/\*/.*/g;
				}
				$users{$hostmask} = $level;
			}
			else
			{
				warn "$hostsfile:$.: unable to parse host file line\n"
			}
		};
		close HOSTS;
	}
	return %users;
}

# order the bot to disconnect
sub do_quit
{
	my ($self) = shift;
	my ($reason);
	my $reasonsfile = sprintf("%s/config/quit.reasons",$self->{'core'}->{'basedir'});
	if (open F,"<" . $reasonsfile)
	{
		my @lines = <F>;
		close(F);
		$reason = $lines[ rand scalar @lines ];
	}
	else
	{
		$reason = "I Quit";
	};
	$self->{'core'}->{'connection'}->quit($reason);
	exit 0;
}

# replies to incoming CTCP PINGs.
sub on_ping {
	my $self = shift;
	my $event = pop @_;
	my $nick = $event->nick;

	$self->{'Core'}->{'connection'}->ctcp_reply($nick, "PING " . join (' ', ($event->args)));
	# Tuxaator, a.k.a. the machine that goes PING!
	$self->{'Core'}->queue($event,"PONG!!!");
}

# reply to CTCP version events
sub on_cversion {
	my ($self,$event) = @_;
	my $version = $self->{'Core'}->{'version'};
	my $netircversion = $self->{'Core'}->{'netircversion'};
	$self->{'Core'}->queue($event,"Tuxaator $version");
	$self->{'Core'}->queue($event,"Net-IRC $netircversion");
	$self->{'Core'}->queue($event,"http://tuxaator.sourceforge.net");
}

1;
