package Users;
use Date::Calc qw(Time_to_Date Delta_DHMS);
# Users plugin, manages the !seen database 
# $Header: /cvsroot/tuxaator/tuxaator/Plugins/Users/Plugin.pm,v 1.14 2004/01/04 19:15:15 duke9 Exp $

sub new {
	my $class = shift;
	my $core = shift;
	my $cfg = shift;
	my $msgs = shift;

	my $self = {
		core => $core,
		cfg => $cfg,
		msgs => $msgs,
	};

	bless $self, $class;
}

sub get_id {
	my $class = shift;
	return "Users";
}

sub get_msg_handlers {
	my $class = shift;
	return ( 
		"!seen"     =>
			{"action"       => "do_seen",
			"restricted"   => 0, },
	);

};

sub get_hooks {
	my $class = shift;
	return {
		"kick" => \&on_kick,
		"nick" => \&on_nick,
		"join" => \&on_join,
		"part" => \&on_part,
		"quit" => \&on_quit,
		353 => \&on_names,
	};
}


# Cycles over the names of people in a channel when we enter and
# toggles their status to "online"
sub on_names {
	my ($self, $event) = @_;
	my (@list, $channel) = ($event->args);
	
	&set_all_users_offline($self);
	
	($channel, @list) = splice @list, 2;
	my (@users,$nick);
	@users = split / /,$list[0];

	foreach $nick (@users) {
		$nick =~ s/^[@|\+]//;	
		$nick = lc $nick;
		#print "nick = $nick\n";
		$self->{'Core'}->{'userlist'}->{$nick} = { "lastmsg" => ""};
		&set_user_status($self,$nick,"1");
	};
}


# when someone leaves the channel, set his or her status to "offline"
sub on_quit {
	my ($self, $event) = @_;
	print $event->nick . " left IRC\n";
	&set_user_status($self,$event->nick,0);
}

# When someone joins, set his or her status to "online"
sub on_join {
	my ($self, $event) = @_;
	my ($channel) = ($event->to)[0];
	my ($nick,$userhost,@lines);
	if ($event->nick ne $self->{'Core'}->{'cfg'}->{'nick'})
	{
		# have to load the lastmsg as well
		# print "user joined!\n";
	}
	else
	{
		$self->{'Core'}->{'curchannel'} = $channel;

		print "bot joined\n";
	};
	print $event->nick . " joined the channel\n";
	&set_user_status($self,$event->nick,1);
}

# When someone leaves a channel the bot is on.
sub on_part {
	my ($self, $event) = @_;
	my $nick = $event->nick;
	my $mynick = $self->{'Core'}->{'cfg'}->{'nick'};
	# if the bot itself is leaving the channel, set all users offline
	if ($nick eq $mynick)
	{
		&set_all_users_offline($self);
	}
	else
	{
		print $event->nick . " left the channel\n";
		&set_user_status($self,$event->nick,0);
	};
}

# catch nick change on the channel and update the status in the database
sub on_nick {
	my ($self, $event) = @_;
	my ($arg) = ($event->args);
	my ($nick) = $event->nick;
	print "*** $nick renamed to to $arg\n";
	&set_user_status($self,$nick,0);
	&set_user_status($self,$arg,1);
};

# catches the kick event, updates user status and prints a message to the channel
sub on_kick {
	my($self,$event) = @_;
	my ($arg) = ($event->args);
	my ($nick) = $event->nick;
	my ($target) = $event->to;
	# if we were kicked, lets try rejoin
	#$self->{'prepend_nick'} = 1;

	if ($target eq $self->{'Core'}->{'cfg'}->{'nick'}) {
		$self->{'Core'}->{'curchannel'} = "";
		sleep(3);
		$self->{'Core'}->{'conn'}->join($self->{'Core'}->{'cfg'}->{'channel'});
	} else {
		&set_user_status($self,$target,0);
		unshift @{$event->to},$self->{'Core'}->{'cfg'}->{'channel'};
	};
};


# finds out when the user was here last time
sub do_seen {
	my $self = shift;
	my $event = pop;
	my @arg = @_;
	my $argstr = join(" ",@arg);
	return unless $argstr;
	my $lc_target = lc $argstr;
	# basic sanity check
	return unless $lc_target =~ /\w/;

	my @slices = qw(86400 3600 60);
	my ($slice,@diff,$mod,@tstr,$msg);

	# kigepealt kontrollime, kas kasutaja on userlistis
	if ($self->{'core'}->{'userlist'}->{$lc_target})
	{
		$status = 1;
		$lastmsg = $self->{'core'}->{'userlist'}->{$argstr}->{'lastmsg'};
		$nick = $argstr;
		$seconds = 0;
	}
	else
	{
		my ($userref) = $self->get_user_status($lc_target,1);
		$status = $userref->{'status'};
		$nick = $userref->{'nick'};
		$seconds = $userref->{'tm'};
		$status = $userref->{'status'};
		$lastmsg = ($userref->{'lastmsg'}) ? $userref->{'lastmsg'} : "n/a";
	};

	if ( ($status) && ($status eq "1") ) {
		if ($lc_target eq lc $event->nick)
		{
			$msg = $self->{'core'}->{'msgs'}->{'seen_self'};
		}
		elsif ($lc_target eq lc $self->{'core'}->{'cfg'}->{'nick'})
		{
			$msg = $self->{'core'}->{'msgs'}->{'seen_bot'};
		}
		else
		{
			$msg = sprintf($self->{'core'}->{'msgs'}->{'seen_target_is_on'},$nick);
		};
	} elsif ($status eq "0") {

		my ($year1,$month1,$day1, $hour1,$min1,$sec1) = Time_to_Date($seconds);
		my ($year2,$month2,$day2, $hour2,$min2,$sec2) = Time_to_Date();

		# now calculate the diff
		my ($Dd,$Dh,$Dm,$Ds) = Delta_DHMS(
			$year1,$month1,$day1, $hour1,$min1,$sec1,
			$year2,$month2,$day2, $hour2,$min2,$sec2);

		if ($Dd > 0)
		{
			push @tstr,sprintf($self->{'core'}->{'msgs'}->{'datediff_days'},$Dd);
		};

		if ($Dh > 0)
		{
			push @tstr,sprintf($self->{'core'}->{'msgs'}->{'datediff_hours'},$Dh);
		};

		if ($Dm > 0)
		{
			push @tstr,sprintf($self->{'core'}->{'msgs'}->{'datediff_minutes'},$Dm);
		};

		if ($Ds > 0)
		{
			push @tstr,sprintf($self->{'core'}->{'msgs'}->{'datediff_seconds'},$Ds);
		};

		my $mask;
		if ($#tstr == 0)
		{
			$mask = $self->{'core'}->{'msgs'}->{'datediff2'};
			$msg = sprintf($mask,$nick,$tstr[$#tstr],$lastmsg);
		}
		else
		{
			$mask = $self->{'core'}->{'msgs'}->{'datediff'};
			$msg = sprintf($mask,$nick,join(":",@{@tstr}[0..$#tstr - 1]),$tstr[$#tstr],$lastmsg);
		};

	} else {
		$msg = sprintf($self->{'core'}->{'msgs'}->{'seen_unknown'},$argstr);
	};
	$self->{'core'}->queue($event,$msg);
	$event->to($self->{'core'}->{'cfg'}->{'channel'});
}

# fetches user status
sub get_user_status {
	my $self = shift;
	my $nick = shift;
	my $fuzzy = shift;
	return unless($nick);

	my ($ref,$seconds,$status,$query,$sth);

	# try with regexp
	$query = "SELECT *,UNIX_TIMESTAMP(timestamp) AS tm FROM users WHERE nick LIKE ? ORDER BY timestamp DESC LIMIT 1";
	$sth = $self->{'core'}->{'dbh'}->prepare($query);
	$sth->execute("%" . $nick . "%");

	# nothing
	if ($sth->rows() == 0)
	{
		return 0;
	}

	# fetch the reference to the record
	$ref = $sth->fetchrow_hashref();

	return $ref;

};

# changes user status to either 1 (on channel) or 0 (not on channel)
sub set_user_status {
	my $self = shift;
	my $nick = shift;
	my $status = shift;

	# this shouldnt happen but we check anyway
	return unless($nick);

	#$nick = &quote($nick);

	$nick =~ s/^@//;
	$nick =~ s/^\+//;

	# first we check whether the user exists in the database
	my $query = "SELECT nick FROM users WHERE nick=?";
	my $sth = $self->{'Core'}->{'dbh'}->prepare($query);


	$sth->execute($nick);

	if ($sth->rows() == 0) {
		# nope, so we add it
		$query = "INSERT INTO users (nick,status) VALUES ('$nick','$status')"
	} else {
		# update 
		$query = "UPDATE users SET status='$status' WHERE nick='$nick'";
	};
	$sth = $self->{'Core'}->{'dbh'}->prepare($query);
	$sth->execute or warn "*** how did this happen? ***\n";

	# if status is going to be zero, then we need to store the users last visible message
	if ($status == 0)
	{
		$lastmsg = $self->{'Core'}->{'dbh'}->quote($self->{'Core'}->{'userlist'}->{$nick}->{'lastmsg'});
		$query = "UPDATE users SET lastmsg = $lastmsg WHERE nick = '$nick'";
		$sth = $self->{'Core'}->{'dbh'}->prepare($query);
		$sth->execute();
		$self->{'Core'}->{'userlist'}->{$nick} = undef;
	}


};

# sets all users offline, called right before bot joins the channel
sub set_all_users_offline {
	my $self = shift;
	my $query = "UPDATE users SET status='0'";
	my $sth = $self->{'Core'}->{'dbh'}->prepare($query);
	$sth->execute();
}

1;
